@@ -109,7 +109,7 @@ void DES_bs_init(int LM)
 
 /* Convert to byte offsets */
 	for (index = 0; index < 0x100; index++)
-		DES_bs_all.s1[index] *= DES_BS_DEPTH >> 3;
+		DES_bs_all.s1[index] *= sizeof(DES_bs_vector);
 
 	if (LM) {
 		for (c = 0; c < 0x100; c++)
@@ -121,6 +121,8 @@ void DES_bs_init(int LM)
 
 #if DES_BS_ASM
 	DES_bs_init_asm();
+#elif defined(__MMX__) || defined(__SSE2__)
+	memset(&DES_bs_all.ones, -1, sizeof(DES_bs_all.ones));
 #endif
 }
 
