@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2005 by Solar Designer
+ * Copyright (c) 1996-2001,2005,2010 by Solar Designer
  */
 
 #include <string.h>
@@ -118,6 +118,16 @@ static int binary_hash_2(void *binary)
 	return *(ARCH_WORD *)binary & 0xFFF;
 }
 
+static int binary_hash_3(void *binary)
+{
+	return *(ARCH_WORD *)binary & 0xFFFF;
+}
+
+static int binary_hash_4(void *binary)
+{
+	return *(ARCH_WORD *)binary & 0xFFFFF;
+}
+
 static int get_hash_0(int index)
 {
 	return DES_bs_get_hash(index, 4);
@@ -133,6 +143,16 @@ static int get_hash_2(int index)
 	return DES_bs_get_hash(index, 12);
 }
 
+static int get_hash_3(int index)
+{
+	return DES_bs_get_hash(index, 16);
+}
+
+static int get_hash_4(int index)
+{
+	return DES_bs_get_hash(index, 20);
+}
+
 static void set_salt(void *salt)
 {
 }
@@ -184,7 +204,9 @@ struct fmt_main fmt_LM = {
 		{
 			binary_hash_0,
 			binary_hash_1,
-			binary_hash_2
+			binary_hash_2,
+			binary_hash_3,
+			binary_hash_4
 		},
 		fmt_default_salt_hash,
 		set_salt,
@@ -195,7 +217,9 @@ struct fmt_main fmt_LM = {
 		{
 			get_hash_0,
 			get_hash_1,
-			get_hash_2
+			get_hash_2,
+			get_hash_3,
+			get_hash_4
 		},
 		cmp_all,
 		cmp_one,
