@@ -38,6 +38,9 @@ extern int CPU_detect(void);
 
 extern struct fmt_main fmt_DES, fmt_BSDI, fmt_MD5, fmt_BF;
 extern struct fmt_main fmt_AFS, fmt_LM;
+#ifdef HAVE_CRYPT
+extern struct fmt_main fmt_crypt;
+#endif
 
 extern int unshadow(int argc, char **argv);
 extern int unafs(int argc, char **argv);
@@ -66,6 +69,9 @@ static void john_register_all(void)
 	john_register_one(&fmt_BF);
 	john_register_one(&fmt_AFS);
 	john_register_one(&fmt_LM);
+#ifdef HAVE_CRYPT
+	john_register_one(&fmt_crypt);
+#endif
 
 	if (!fmt_list) {
 		fprintf(stderr, "Unknown ciphertext format name requested\n");
@@ -281,7 +287,7 @@ static void john_run(void)
 				log_event("- Configured to use otherwise idle "
 					"processor cycles only");
 		}
-		tty_init();
+		tty_init(options.flags & FLG_STDIN_CHK);
 
 		if (options.flags & FLG_SINGLE_CHK)
 			do_single_crack(&database);
