@@ -19,6 +19,10 @@
 #include "formats.h"
 #include "loader.h"
 
+#ifdef HAVE_CRYPT
+extern struct fmt_main fmt_crypt;
+#endif
+
 /*
  * Flags for read_file().
  */
@@ -34,6 +38,8 @@
 static char issep_map[0x100];
 static int issep_initialized = 0;
 
+static char *no_username = "?";
+
 static void read_file(struct db_main *db, char *name, int flags,
 	void (*process_line)(struct db_main *db, char *line))
 {
@@ -216,7 +222,12 @@ static int ldr_split_line(char **login, char **ciphertext,
 	*login = ldr_get_field(&line);
 	if (!strcmp(*login, "+") || !strncmp(*login, "+@", 2)) return 0;
 
-	if (!*(*ciphertext = ldr_get_field(&line))) if (!line) return 0;
+	if (!*(*ciphertext = ldr_get_field(&line)) && !line) {
+		if (strlen(*login) < 13)
+			return 0;
+		*ciphertext = *login;
+		*login = no_username;
+	}
 
 	if (source) strcpy(source, line ? line : "");
 
@@ -254,6 +265,23 @@ static int ldr_split_line(char **login, char **ciphertext,
 
 	if ((*format = fmt_list))
 	do {
+#ifdef HAVE_CRYPT
+/*
+ * Only probe for support by the current system's crypt(3) if this is forced
+ * from the command-line or/and if the hash encoding string looks like one of
+ * those that are only supported in that way.  Avoid the probe in other cases
+ * because it may be slow and undesirable (false detection is possible).
+ */
+		if (*format == &fmt_crypt &&
+		    fmt_list != &fmt_crypt /* not forced */ &&
+#ifdef __sun
+		    strncmp(*ciphertext, "$md5$", 5) &&
+		    strncmp(*ciphertext, "$md5,", 5) &&
+#endif
+		    strncmp(*ciphertext, "$5$", 3) &&
+		    strncmp(*ciphertext, "$6$", 3))
+			continue;
+#endif
 		if ((count = (*format)->methods.valid(*ciphertext))) {
 			fmt_init(*format);
 			return count;
@@ -290,10 +318,11 @@ static struct list_main *ldr_init_words(char *login, char *gecos, char *home)
 
 	list_init(&words);
 
-	if (*login)
+	if (*login && login != no_username)
 		list_add(words, login);
 	ldr_split_string(words, gecos);
-	ldr_split_string(words, login);
+	if (login != no_username)
+		ldr_split_string(words, login);
 
 	if ((pos = strrchr(home, '/')) && pos[1])
 		list_add_unique(words, pos + 1);
@@ -421,6 +450,9 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 				sprintf(current_pw->login, "%s:%d",
 					login, index + 1);
 			} else
+			if (login == no_username)
+				current_pw->login = login;
+			else
 			if (words && *login)
 				current_pw->login = words->head->data;
 			else
