@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99,2003,2004,2006 by Solar Designer
+ * Copyright (c) 1996-99,2003,2004,2006,2010 by Solar Designer
  */
 
 #include <stdio.h>
@@ -125,35 +125,62 @@ static void single_init(void)
 
 static int single_key_hash(char *key)
 {
-	int pos, hash = 0;
+	unsigned int hash, extra, pos;
 
-	for (pos = 0; pos < length && *key; pos++) {
-		hash <<= 1;
-		hash ^= *key++;
+	hash = (unsigned char)key[0];
+	if (!hash)
+		goto out;
+	extra = (unsigned char)key[1];
+	if (!extra)
+		goto out_and;
+
+	pos = 3;
+	if (length & 1) {
+		while (key[2]) {
+			hash += (unsigned char)key[2];
+			if (!key[3] || pos >= length) break;
+			extra += (unsigned char)key[3];
+			key += 2;
+			pos += 2;
+		}
+	} else {
+		while (key[2] && pos < length) {
+			hash += (unsigned char)key[2];
+			if (!key[3]) break;
+			extra += (unsigned char)key[3];
+			key += 2;
+			pos += 2;
+		}
 	}
 
-	hash ^= hash >> SINGLE_HASH_LOG;
-	hash ^= hash >> (2 * SINGLE_HASH_LOG);
-	hash &= SINGLE_HASH_SIZE - 1;
+	hash -= extra + pos;
+#if SINGLE_HASH_LOG > 6
+	hash ^= extra << 6;
+#endif
 
+out_and:
+	hash &= SINGLE_HASH_SIZE - 1;
+out:
 	return hash;
 }
 
 static int single_add_key(struct db_keys *keys, char *key)
 {
-	int index, hash;
+	int index, new_hash, reuse_hash;
 	struct db_keys_hash_entry *entry;
 
-	if ((index = keys->hash->hash[single_key_hash(key)]) >= 0)
+/* Check if this is a known duplicate, and reject it if so */
+	if ((index = keys->hash->hash[new_hash = single_key_hash(key)]) >= 0)
 	do {
 		entry = &keys->hash->list[index];
 		if (!strncmp(key, &keys->buffer[entry->offset], length))
 			return 0;
 	} while ((index = entry->next) >= 0);
 
-	index = keys->hash->hash[hash = single_key_hash(keys->ptr)];
+/* Update the hash table removing the list entry we're about to reuse */
+	index = keys->hash->hash[reuse_hash = single_key_hash(keys->ptr)];
 	if (index == keys->count)
-		keys->hash->hash[hash] = keys->hash->list[index].next;
+		keys->hash->hash[reuse_hash] = keys->hash->list[index].next;
 	else
 	if (index >= 0) {
 		entry = &keys->hash->list[index];
@@ -166,11 +193,12 @@ static int single_add_key(struct db_keys *keys, char *key)
 		}
 	}
 
-	index = keys->hash->hash[hash = single_key_hash(key)];
+/* Add the new entry */
+	index = keys->hash->hash[new_hash];
 	entry = &keys->hash->list[keys->count];
 	entry->next = index;
 	entry->offset = keys->ptr - keys->buffer;
-	keys->hash->hash[hash] = keys->count;
+	keys->hash->hash[new_hash] = keys->count;
 
 	strnfcpy(keys->ptr, key, length);
 	keys->ptr += length;
@@ -186,6 +214,18 @@ static int single_process_buffer(struct db_salt *salt)
 
 	if (crk_process_salt(salt)) return 1;
 
+/*
+ * Flush the keys list (since we've just processed the keys), but not the hash
+ * table to allow for more effective checking for duplicates.  We could flush
+ * the hash table too, such as by calling single_alloc_keys() here, which would
+ * allow us to drop the update-hash-before-list-entry-reuse code from
+ * single_add_key().  This would speed things up in terms of this source file's
+ * code overhead, however it would allow more duplicates to pass.  The apparent
+ * c/s rate (counting duplicates as if they were distinct combinations) would
+ * be higher, but the number of passwords cracked per unit of time might be
+ * lower or higher depending on many things including the relative speed of
+ * password hash computations vs. the "overhead".
+ */
 	keys = salt->keys;
 	keys->count = 0;
 	keys->ptr = keys->buffer;
@@ -230,16 +270,19 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 	char *key;
 	unsigned int c;
 
+	if (!(first = pw->words->head))
+		return -1;
+
 	keys = salt->keys;
 
 	first_number = 0;
-	if ((first = pw->words->head))
 	do {
 		if ((key = rules_apply(first->data, rule, 0, NULL)))
 		if (ext_filter(key))
 		if (single_add_key(keys, key))
 		if (single_process_buffer(salt)) return 1;
-		if (!salt->list) return 0;
+		if (!salt->list) return 2;
+		if (!pw->binary) return 0;
 
 		if (++first_number > SINGLE_WORDS_PAIR_MAX) continue;
 
@@ -259,7 +302,8 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 				if (ext_filter(key))
 				if (single_add_key(keys, key))
 				if (single_process_buffer(salt)) return 1;
-				if (!salt->list) return 0;
+				if (!salt->list) return 2;
+				if (!pw->binary) return 0;
 			}
 
 			if (first->data[1]) {
@@ -271,7 +315,8 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 				if (ext_filter(key))
 				if (single_add_key(keys, key))
 				if (single_process_buffer(salt)) return 1;
-				if (!salt->list) return 0;
+				if (!salt->list) return 2;
+				if (!pw->binary) return 0;
 			}
 		} while (++second_number <= SINGLE_WORDS_PAIR_MAX &&
 			(second = second->next));
@@ -283,14 +328,35 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 static int single_process_salt(struct db_salt *salt, char *rule)
 {
 	struct db_keys *keys;
-	struct db_password *pw;
+	struct db_password *pw, **last;
+	int status, have_words = 0;
 
 	keys = salt->keys;
+	if (!keys)
+		return 0;
 
-	pw = salt->list;
+	last = &salt->list;
+	pw = *last;
 	do {
-		if (single_process_pw(salt, pw, rule)) return 1;
-		if (!salt->list) return 0;
+/*
+ * "binary" is set to NULL on entries marked for removal (so we remove them
+ * here) or already removed (yet we might hit them once in some obscure cases).
+ */
+		if (pw->binary) {
+			if (!(status = single_process_pw(salt, pw, rule))) {
+				have_words = 1;
+				goto next;
+			}
+			if (status < 0) /* no words for this hash */
+				goto next;
+			if (status == 2) /* no hashes left for this salt */
+				return 0;
+			return 1; /* no hashes left to crack for all salts */
+		} else {
+			*last = pw->next; /* remove */
+		}
+next:
+		last = &pw->next;
 	} while ((pw = pw->next));
 
 	if (keys->count && rule_number - keys->rule > (key_count << 1))
@@ -298,6 +364,11 @@ static int single_process_salt(struct db_salt *salt, char *rule)
 
 	if (!keys->count) keys->rule = rule_number;
 
+	if (!have_words) {
+		if (keys->count && single_process_buffer(salt)) return 1;
+		salt->keys = NULL;
+	}
+
 	return 0;
 }
 
@@ -306,10 +377,11 @@ static void single_run(void)
 	char *prerule, *rule;
 	struct db_salt *salt;
 	int min, saved_min;
+	int have_words;
 
 	saved_min = rec_rule;
 	while ((prerule = rpp_next(rule_ctx))) {
-		if (!(rule = rules_reject(prerule, 0, single_db))) {
+		if (!(rule = rules_reject(prerule, 0, NULL, single_db))) {
 			log_event("- Rule #%d: '%.100s' rejected",
 				++rule_number, prerule);
 			continue;
@@ -328,17 +400,28 @@ static void single_run(void)
 			saved_min = rec_rule;
 		}
 
+		have_words = 0;
+
 		min = rule_number;
 
 		salt = single_db->salts;
 		do {
 			if (!salt->list) continue;
 			if (single_process_salt(salt, rule)) return;
-			if (salt->keys->rule < min) min = salt->keys->rule;
+			if (!salt->keys) continue;
+			have_words = 1;
+			if (salt->keys->rule < min)
+				min = salt->keys->rule;
 		} while ((salt = salt->next));
 
 		rec_rule = min;
 		rule_number++;
+
+		if (have_words) continue;
+
+		log_event("- No information to base%s candidate passwords on",
+			rule_number > 1 ? " further" : "");
+		return;
 	}
 }
 
@@ -349,10 +432,10 @@ static void single_done(void)
 	if (!event_abort) {
 		if ((salt = single_db->salts)) {
 			log_event("- Processing the remaining buffered "
-				"candidate passwords");
+				"candidate passwords, if any");
 
 			do {
-				if (!salt->list) continue;
+				if (!salt->list || !salt->keys) continue;
 				if (salt->keys->count)
 				if (single_process_buffer(salt)) break;
 			} while ((salt = salt->next));
