@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 2003,2006,2008 by Solar Designer
+ * Copyright (c) 2003,2006,2008,2010 by Solar Designer
  */
 
 /*
@@ -32,11 +32,32 @@
 #define DES_SCALE			1
 #define DES_EXTB			1
 #define DES_COPY			0
-#define DES_BS_ASM			1
-#define DES_BS				2
+#if defined(__SSE2__) && \
+    ((__GNUC__ == 4 && __GNUC_MINOR__ >= 4) || __GNUC__ > 4)
+#define DES_BS_ASM			0
+#if 1
 #define DES_BS_VECTOR			2
-#define DES_BS_EXPAND			1
 #define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2-16"
+#elif 0
+#define DES_BS_VECTOR			3
+#define DES_BS_VECTOR_SIZE		4
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2-16 + 64/64 BS MMX"
+#elif 0
+#define DES_BS_NO_MMX
+#define DES_BS_VECTOR			3
+#define DES_BS_VECTOR_SIZE		4
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2-16 + 64/64 BS"
+#else
+#define DES_BS_VECTOR			4
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2-16 + 64/64 BS MMX + 64/64 BS"
+#endif
+#else
+#define DES_BS_ASM			1
+#define DES_BS_VECTOR			2
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2-16"
+#endif
+#define DES_BS				1
+#define DES_BS_EXPAND			1
 
 #define MD5_ASM				0
 #define MD5_X2				1
