@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2002,2005,2006,2008 by Solar Designer
+ * Copyright (c) 1996-2002,2005,2006,2008,2010 by Solar Designer
  */
 
 /*
@@ -45,18 +45,43 @@
 #define DES_EXTB			0
 #define DES_COPY			1
 #define DES_STD_ALGORITHM_NAME		"48/64 4K MMX"
-#define DES_BS_ASM			1
-#define DES_BS				1
+#if defined(__SSE2__) && 0
+#define DES_BS_ASM			0
+#if 1
 #define DES_BS_VECTOR			4
-#define DES_BS_EXPAND			1
 #define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2"
+#elif 0
+#define DES_BS_VECTOR			6
+#define DES_BS_VECTOR_SIZE		8
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2 + 64/64 BS MMX"
+#elif 0
+#define DES_BS_VECTOR			5
+#define DES_BS_VECTOR_SIZE		8
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2 + 32/32 BS"
+#else
+#define DES_BS_VECTOR			7
+#define DES_BS_VECTOR_SIZE		8
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2 + 64/64 BS MMX + 32/32 BS"
+#endif
+#else
+#define DES_BS_ASM			1
+#define DES_BS_VECTOR			4
+#define DES_BS_ALGORITHM_NAME		"128/128 BS SSE2"
+#endif
+#define DES_BS				1
+#define DES_BS_EXPAND			1
 
 #define MD5_ASM				1
 #define MD5_X2				0
 #define MD5_IMM				1
 
+#ifdef _OPENMP
+#define BF_ASM				0
+#define BF_X2				1
+#else
 #define BF_ASM				1
-#define BF_SCALE			1
 #define BF_X2				0
+#endif
+#define BF_SCALE			1
 
 #endif
